(***********************************************************************)
(*                                                                     *)
(*                           HWeak                                     *)
(*                                                                     *)
(*                        Remi Vanicat                                 *)
(*                                                                     *)
(*  Copyright 2002 Rmi Vanicat                                        *)
(*  All rights reserved.  This file is distributed under the terms of  *)
(*  the GNU Library General Public License, with the special exception *)
(*  on linking described in the LICENCE file of the Objective Caml     *)
(*  distribution                                                       *)
(*                                                                     *)
(*  The original Idea come from the GUtil.memo class of the lablgtk    *)
(*  library                                                            *)
(*                                                                     *)
(***********************************************************************)

(** The memo class provides an easy way to remember the real class of
  an object.
  
  This module contain the [Weak_memo.c] class that contain object that 
  can be recover latter from the same object with a different type. An
  object that is into a [Weak_memo.c] object is not prevented from
  recolection by the Gc. *)

(** The memo class.

  [new Weak_memo.c size] create a new memo object with an original size
  of [size]. It will grow as needed. *)
class ['a] c : int ->
object
  constraint 'a = < .. >

  (** Add an object to the memo. This won't prevent this object to be
    recollected by the Gc.
    
    Do nothing if the object is already there. *)
  method add : 'a -> unit

  (** Find if the object is in the memo. If yes then return it (with the good 
    type).

    Otherwise raise [Not_found] *)
  method find : < .. >  -> 'a

  (** Return true if there is an object in the memo that is equal to the
    object given as argument. Return false otherwise. *)
  method mem : < .. > -> bool

  (** Remove an object from the memo.
    If it is not there, do nothing *)
  method remove : < .. > -> unit

  (** Empty the memo object. *)
  method clear : unit -> unit

  (** Return the number of object in the memo. *)
  method count : int
    
end
